/* This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// BusSchedWCEDlg.cpp : implementation file
//

#include "stdafx.h"
//#ifdef _WIN32_WCE
//#include <stdlib.h>    /* For atoi(). */
//#endif
#include "BusSchedWCE.h"
#include "BusSchedWCEDlg.h"
#include "AboutDlg.h"
#include "PrefsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBusSchedWCEDlg dialog

CBusSchedWCEDlg::CBusSchedWCEDlg(CWnd* pParent /*=NULL*/)
   : CDialog(CBusSchedWCEDlg::IDD, pParent)
{
   //{{AFX_DATA_INIT(CBusSchedWCEDlg)
      // NOTE: the ClassWizard will add member initialization here
   //}}AFX_DATA_INIT
   // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

    // Ensure everything is initialized to valid values that are referenced later.
    m_bFirstPaint = TRUE;    // Set to FALSE in first OnPaint() call.
    m_lpdbFileData = NULL;
    m_lpDbInfo = NULL;
    m_uNextBusOffset = NEXT;
    m_dwDbFileLength = 0;

    CWinApp* pApp = AfxGetApp();
    m_dbFileName = pApp->GetProfileString( INI_SECTION, INI_DBFILENAME );
    m_uRoute = pApp->GetProfileInt( INI_SECTION, INI_ROUTE, 0 );
    bDirection = pApp->GetProfileInt( INI_SECTION, INI_DIRECTION, DIRECTION_OUT );
    m_uMinsFromTerminus = pApp->GetProfileInt( INI_SECTION, INI_MINSFROMTERMINUS, 0 );

    ClearRouteArray();
}

void CBusSchedWCEDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   //{{AFX_DATA_MAP(CBusSchedWCEDlg)
   DDX_Control(pDX, IDC_TIME_LIST, m_Timetable);
   DDX_Control(pDX, IDC_ROUTE_COMBO, m_RouteCombo);
   DDX_Control(pDX, IDC_DAY_COMBO, m_DayCombo);
   //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CBusSchedWCEDlg, CDialog)
   //{{AFX_MSG_MAP(CBusSchedWCEDlg)
   ON_CBN_SELCHANGE(IDC_ROUTE_COMBO, OnSelchangeRouteCombo)
   ON_CBN_SELCHANGE(IDC_DAY_COMBO, OnSelchangeDayCombo)
   ON_BN_CLICKED(IDC_EARLIER_BTN, OnEarlierBtn)
   ON_BN_CLICKED(IDC_INFO_BTN, OnInfoBtn)
   ON_BN_CLICKED(IDC_LATER_BTN, OnLaterBtn)
   ON_BN_CLICKED(IDC_REVERSE_BTN, OnReverseBtn)
   ON_WM_LBUTTONDOWN()
   ON_COMMAND(IDS_SELECT_TIMETABLE, OnSelectTimetable)
   ON_COMMAND(IDS_ABOUTBOX, OnAboutbox)
   ON_COMMAND(IDS_PREFERENCES, OnPreferences)
   ON_WM_PAINT()
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBusSchedWCEDlg message handlers

BOOL CBusSchedWCEDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   // Set the icon for this dialog.  The framework does this automatically
   //  when the application's main window is not a dialog
   SetIcon(m_hIcon, TRUE);         // Set big icon
   SetIcon(m_hIcon, FALSE);      // Set small icon
   
   CenterWindow(GetDesktopWindow());   // center to the hpc screen

   // TODO: Add extra initialization here
    // Add 3 columns.
    LV_COLUMN lvc;
    lvc.mask = LVCF_WIDTH;
    // Split table into 3 even columns.
    RECT rect;
    m_Timetable.GetClientRect( &rect );
    lvc.cx = rect.right / TABLE_COLUMNS;

    // Add the columns. One column implied so add cols 2 and 3.
    for ( UINT col = 1; col <= TABLE_COLUMNS; col++ )
    {
        lvc.iSubItem = col;
        m_Timetable.InsertColumn( 0,&lvc );
    }

    // Add items.
    // Determine number of rows in the table.
    UINT nRows = m_Timetable.GetCountPerPage();
    for ( col = 0; col < TABLE_COLUMNS; col++ )
    {
        LVITEM lvi;
        lvi.mask = LVIF_TEXT;
        lvi.iSubItem = col;

        for ( UINT row = 0; row < nRows; row++ )
        {
            lvi.iItem = row;
            lvi.pszText = _T( "" );
            lvi.cchTextMax = sizeof( _T( "" ) );
            m_Timetable.InsertItem( &lvi );
        }
    }


    // Load the days into the day combo box.
    CString csDay;
    INT nIndex;
    csDay.LoadString( IDS_MF );
    nIndex = m_DayCombo.AddString( csDay );
    // Set the item data to the string ID for use in messages later.
    m_DayCombo.SetItemData( nIndex, IDS_MF );
    csDay.LoadString( IDS_SAT );
    nIndex = m_DayCombo.AddString( csDay );
    m_DayCombo.SetItemData( nIndex, IDS_SAT );
    csDay.LoadString( IDS_SUN );
    nIndex = m_DayCombo.AddString( csDay );
    m_DayCombo.SetItemData( nIndex, IDS_SUN );

    // Set the day combo box selection based on the current day of the week.
    CTime cCtime = CTime::GetCurrentTime();
    switch ( cCtime.GetDayOfWeek() )
    {
        case 1:  m_nDayCombo = 2; break;    // Sunday.
        case 7:  m_nDayCombo = 1; break;    // Saturday.
        default: m_nDayCombo = 0; break;    // Mon-Fri.
    }
    m_DayCombo.SetCurSel( m_nDayCombo );
   
    return TRUE;  // return TRUE  unless you set the focus to a control
}


BOOL CBusSchedWCEDlg::ReadTimetableFile()
{
    CFile dbFile;
    CFileException e;

    // Ensure the filename is set.
    if ( m_dbFileName.GetLength() == 0 )
        return FALSE;

    if ( ! dbFile.Open( m_dbFileName, CFile::modeRead | CFile::shareDenyWrite, &e ) )
    {
        CString sErrorMsg;
        e.GetErrorMessage( sErrorMsg.GetBuffer( 1024 ), 1024 );
        sErrorMsg.ReleaseBuffer();
        CString sTitle( (LPCSTR) IDS_BUSSCHED_ERROR );
        CString sMsg( (LPCSTR) IDS_SELECT_ANOTHER_FILE );
        sErrorMsg += "\n" + sMsg;
        MessageBox( sErrorMsg, sTitle );

        OnSelectTimetable();

        if ( ReadTimetableFile() )
        {
            // Display first route and default direction as new timetable loaded.
            m_uRoute = 0;
            bDirection = DIRECTION_OUT;
            ReloadTimetable();
         }

        return FALSE;
    }

    // TODO: This code should never run.
    if ( CFile::hFileNull == dbFile.m_hFile )
    {
        // TODO: Put this message in stringtable.
        MessageBox( _T( "ERROR: Unable to open timetable file." ) );
        return FALSE;
    }

    
    // Remove the NULL chars from m_lpdbFileData so delete() releases whole block.
    if ( m_lpdbFileData != NULL )
    {
        memset( m_lpdbFileData, 'x', m_dwDbFileLength );
        ::delete( m_lpdbFileData );
    }

    m_dwDbFileLength = dbFile.GetLength();
    m_lpdbFileData = (LPTSTR) new TCHAR [ m_dwDbFileLength + 1 ];
    dbFile.Read( m_lpdbFileData, m_dwDbFileLength );
    // TODO: This fails on PocketPC!!
    m_lpdbFileData[ m_dwDbFileLength ] = NULL;

    // Done with the file so close it.
    if ( dbFile.m_hFile != CFile::hFileNull )
        dbFile.Close();

    UINT i = 0;
    UINT nLine = 0;
    LPTSTR lpCurrent = m_lpdbFileData;
    LPTSTR lpNull;

#ifdef _UNICODE
    // Unicode text files start with 0xFFEE so move over that byte.
    if ( *lpCurrent == 0xFEFF )
        lpCurrent++;
    else
    {
        // Convert non-Unicode data to Unicode.
        LPWSTR lpwdbFileData = NULL;
        // Find out how many chars needed when data converted to wide chars.
        INT nBuffSize = MultiByteToWideChar( CP_ACP, MB_COMPOSITE, (LPCSTR)m_lpdbFileData, m_dwDbFileLength, lpwdbFileData, 0 );
        // Allocate necessary buffer.
        lpwdbFileData = (LPWSTR) new TCHAR [ nBuffSize + 1 ];
        // Do conversion of ANSI data to wide chars.
        MultiByteToWideChar( CP_ACP, MB_COMPOSITE, (LPCSTR)m_lpdbFileData, m_dwDbFileLength, lpwdbFileData, nBuffSize );
        // Free current buffer and point current buffer to new buffer.
        ::delete( m_lpdbFileData );
        m_lpdbFileData = lpwdbFileData;
        lpCurrent = m_lpdbFileData;
    }
#endif

    while ( ( i < MAX_ROUTES ) && ( lpCurrent <= ( m_lpdbFileData + m_dwDbFileLength ) ) )
    {
        nLine = 0;

        while ( nLine < ROUTE_REC_SIZE )
        {
            m_aRoutes[ i ].lpLine[ nLine ] = lpCurrent;

            // When processing last line (db info) will hit NULL but not 0x0D.
            while ( ( *lpCurrent ) && ( *lpCurrent != 0x0D ) )
                lpCurrent++;
            if ( *lpCurrent == NULL )
            {
                // Store the db info record in another variable.
                m_lpDbInfo = m_aRoutes[ i ].lpLine[ nLine ];
                m_aRoutes[ i ].lpLine[ nLine ] = NULL;
                break;
            }

            lpNull = lpCurrent;
            lpCurrent++;
            *lpNull = NULL;
            if ( *lpCurrent == 0x0A )
                lpCurrent++;
            nLine++;
        }

        if ( *lpCurrent != NULL )
        {
            m_RouteCombo.AddString( m_aRoutes[ i ].lpLine[ 0 ] );
            // Since at least one route listed, enable both route and day combos and buttons.
            m_RouteCombo.EnableWindow();
            m_DayCombo.EnableWindow();
            GetDlgItem( IDC_REVERSE_BTN )->EnableWindow();
            GetDlgItem( IDC_INFO_BTN )->EnableWindow();
        }
        else
            break;

        i++;
    }

    m_RouteCombo.SetCurSel( 0 );

    lpCurrent = NULL;
    lpNull = NULL;

    return TRUE;
}

// Set the route to NULL for each element in the route array.
void CBusSchedWCEDlg::ClearRouteArray()
{
    for ( UINT i = 0; i < MAX_ROUTES; i++ )
        for ( UINT j = 0; j < ROUTE_REC_SIZE; j++ )
            m_aRoutes[ i ].lpLine[ j ] = NULL;
}

void CBusSchedWCEDlg::PostNcDestroy() 
{
    // Remove the NULL chars from m_lpdbFileData so delete() releases whole block.
    if ( m_lpdbFileData != NULL )
    {
        memset( m_lpdbFileData, 'x', m_dwDbFileLength );
        ::delete( m_lpdbFileData );
    }

    // Store preferences.
    CWinApp* pApp = AfxGetApp();
    pApp->WriteProfileString( INI_SECTION, INI_DBFILENAME, m_dbFileName );
    CString csPrefStr;
    csPrefStr.Format( _T( "%u" ), m_uRoute );
    pApp->WriteProfileString( INI_SECTION, INI_ROUTE, csPrefStr );
    csPrefStr.Format( _T( "%d" ), bDirection );
    pApp->WriteProfileString( INI_SECTION, INI_DIRECTION, csPrefStr );
    csPrefStr.Format( _T( "%u" ), m_uMinsFromTerminus );
    pApp->WriteProfileString( INI_SECTION, INI_MINSFROMTERMINUS, csPrefStr );

    CDialog::PostNcDestroy();
}

void CBusSchedWCEDlg::OnSelchangeRouteCombo() 
{
    m_uRoute = m_RouteCombo.GetCurSel();

    /* Looking at a new route so default to viewing next buses (based on current time). */
    m_uNextBusOffset = NEXT;
    ReloadTimetable();
}

void CBusSchedWCEDlg::OnSelchangeDayCombo() 
{
    m_nDayCombo = m_DayCombo.GetCurSel();

    /* Looking at a new day so default to viewing next buses (based on current time). */
    m_uNextBusOffset = NEXT;
    ReloadTimetable();
}

void CBusSchedWCEDlg::OnReverseBtn() 
{
    // If no timetable loaded then call function to select one.
    if ( ! m_dbFileName.GetLength() )
    {
        OnSelectTimetable();
        return;
    }

    bDirection = !bDirection;
    /* Looking at a new direction so default to viewing next buses (based on current time). */
    m_uNextBusOffset = NEXT;
 
    ReloadTimetable();
}

void CBusSchedWCEDlg::OnInfoBtn() 
{
    // If no timetable loaded then call function to select one.
    if ( ! m_dbFileName.GetLength() )
    {
        OnSelectTimetable();
        return;
    }

    CString csTitle;
    csTitle.Format( IDS_INFO_TITLE, m_aRoutes[ m_uRoute ].lpLine[ ROUTE_NUM_OFFSET ] );
    CString csInfo( ( LPCTSTR ) IDS_NO_ROUTE_INFO );    // Default to 'No more info' type string.
    if ( lstrlen( m_aRoutes[ m_uRoute ].lpLine[ ROUTE_INFO_OFFSET ] ) )
        csInfo = m_aRoutes[ m_uRoute ].lpLine[ ROUTE_INFO_OFFSET ];
    csInfo.Replace( '|', '\n' );    // Change pipe chars to carriage returns.
    MessageBox( csInfo, csTitle );   
}

void CBusSchedWCEDlg::OnEarlierBtn() 
{
    // If no timetable loaded then call function to select one.
    if ( ! m_dbFileName.GetLength() )
    {
        OnSelectTimetable();
        return;
    }

    /* Reset the offset so the first buses of the day are shown. */
    m_uNextBusOffset = FIRST;
    ReloadTimetable();
}

void CBusSchedWCEDlg::OnLaterBtn() 
{
    // If no timetable loaded then call function to select one.
    if ( ! m_dbFileName.GetLength() )
    {
        OnSelectTimetable();
        return;
    }

    /* The offset will be used to view the next batch of buses. */
    ReloadTimetable();
}

void CBusSchedWCEDlg::ReloadTimetable()
{
    // m_uRoute must be less than the number of routes in the combo box.
    VERIFY ( m_uRoute < (UINT) m_RouteCombo.GetCount() );

    if ( bDirection )
    {
        SetDlgItemText( IDC_FROM, m_aRoutes[ m_uRoute ].lpLine[ FROM_OFFSET ] );
        SetDlgItemText( IDC_TO, m_aRoutes[ m_uRoute ].lpLine[ TO_OFFSET ] );
    }
    else
    {
        SetDlgItemText( IDC_FROM, m_aRoutes[ m_uRoute ].lpLine[ TO_OFFSET ] );
        SetDlgItemText( IDC_TO, m_aRoutes[ m_uRoute ].lpLine[ FROM_OFFSET ] );
    }


    // Determine number of rows in the table.
    UINT nRows = m_Timetable.GetCountPerPage();
    // Clear table
    UINT col, row;
    for ( col = 0; col < TABLE_COLUMNS; col++ )
    {
        LVITEM lvi;
        lvi.mask = LVIF_TEXT;
        lvi.iSubItem = col;

        for ( row = 0; row < nRows; row++ )
        {
            lvi.iItem = row;
            lvi.pszText = _T( "" );
            lvi.cchTextMax = sizeof( _T( "" ) );
            m_Timetable.SetItem( &lvi );
        }
    }


    /* Get the current time, less offset. */
    CTime cCtime = CTime::GetCurrentTime();
    // Deduct m_uMinsFromTerminus to look at buses leaving that many mins ago.
    cCtime -= CTimeSpan( 0, 0, m_uMinsFromTerminus, 0 );
    UINT uCurrentTime = cCtime.GetHour() * 100 + cCtime.GetMinute();
#ifndef _WIN32_WCE
    LCID lcid = GetThreadLocale();
#endif
    TCHAR szFormattedTime[ 20 ];
    UINT uNextBusTime = 0;

    // Set different times to show app works.
    // TODO: Make lpTimes a member variable.
    LPTSTR lpTimes;
    if ( bDirection )
        lpTimes = m_aRoutes[ m_uRoute ].lpLine[ m_nDayCombo * 2 + MF_OUT_TIMES_OFFSET ];
    else
        lpTimes = m_aRoutes[ m_uRoute ].lpLine[ m_nDayCombo * 2 + MF_BACK_TIMES_OFFSET ];

    LPTSTR lpStart, lpEnd;
    switch ( m_uNextBusOffset )
    {
        case FIRST:
            lpStart = lpTimes;
            lpEnd = lpTimes;
            /* Viewing the first buses of the day so hide 'Earlier' button. */
            GetDlgItem( IDC_EARLIER_BTN )->EnableWindow( FALSE );
            break;
        case NEXT:
            lpStart = lpTimes;
            lpEnd = lpTimes;
            /* Find the next bus by looking for a bus at or after the current time. */
            uNextBusTime = 0;

            while ( *lpStart  && ( uCurrentTime > uNextBusTime ) )
            {
                uNextBusTime = _ttoi( lpStart );    // _ttoi is wide version of atoi.
                if ( ( uCurrentTime > uNextBusTime ) && *lpStart )
                {
                    // Move to the next delimiter.
                    lpStart = CharNext( lpStart );
                    // Move beyond the delimiter.
                    if ( *lpStart )
                        lpStart = CharNext( lpStart );
                }
            }

            lpEnd = lpStart;

            if ( lpTimes == lpStart )
                /* Already viewing the first buses so hide the 'Earlier' button. */
                GetDlgItem( IDC_EARLIER_BTN )->EnableWindow( FALSE );
            else
                /* Not viewing first buses so show the 'Earlier' button. */
                GetDlgItem( IDC_EARLIER_BTN )->EnableWindow( TRUE );
            break;
        default:
            /* Begin a certain offset into the string. Used when 'Later' button pressed. */
            lpStart = lpTimes + m_uNextBusOffset;
            lpEnd = lpStart;
            /* Not viewing first buses so show the 'Earlier' button. */
            GetDlgItem( IDC_EARLIER_BTN )->EnableWindow( TRUE );
            break;
    }

    /* Count the number of bus times displayed to determine whether */
    /* a 'Next' or 'More' button is needed. */
    UINT uNumBuses = 0;
    SYSTEMTIME stTime;
    // Populate struct with valid values in case GetTimeFormat() uses them. */
    GetLocalTime( &stTime );

    // Populate table.
    for ( col = 0; col < TABLE_COLUMNS; col++ )
    {
        TCHAR szDepartTime[ 10 ];
        UINT nDepartTime;
        LVITEM lvi;
        lvi.mask = LVIF_TEXT;
        lvi.iSubItem = col;

        LPTSTR lpAnnotation;
        for ( row = 0; row < nRows; row++ )
        {
            while ( *lpEnd  && *lpEnd != TIME_SEP_CHAR )
                lpEnd = CharNext( lpEnd );
            /* Break out if at the end of the string. */
            if ( ! *lpEnd )
                break;
#ifndef _WIN32_WCE
            lstrcpyn( szDepartTime, lpStart, lpEnd + 1 - lpStart );
#else
            // lstrcpyn() is not available on Pocket PC.
            for ( INT i = 0; i <= lpEnd + 1 - lpStart; i++ )
            {
                *( szDepartTime + i ) = *( lpStart + i );
            }
#endif
            szDepartTime[ lpEnd - lpStart ] = '\0';
            nDepartTime = _ttoi( szDepartTime );
            stTime.wHour = nDepartTime / 100;
            stTime.wMinute = nDepartTime % 100;
            /* Format time, without seconds, in user's chosen locale format. */
            /* For Pocket PC force 24 hour because screen too narrow for AM/PM and annotations. */
            GetTimeFormat( NULL, TIME_NOSECONDS | TIME_FORCE24HOURFORMAT, &stTime, _T( "H':'mm" )/*NULL*/, szFormattedTime, sizeof( szFormattedTime ) );
            lvi.iItem = row;

            // Find and append annotation, if any.
            lpAnnotation = szDepartTime;
            while ( lpAnnotation && *lpAnnotation >= L'0' && *lpAnnotation <= L'9' )
                lpAnnotation++;
            if ( lpAnnotation )
            {
                lstrcat( szFormattedTime, _T( " " ) );
                lstrcat( szFormattedTime, lpAnnotation );
            }
            // Add string to table cell.
            lvi.pszText = szFormattedTime;
            lvi.cchTextMax = lstrlen( szFormattedTime );
            m_Timetable.SetItem( &lvi );
            uNumBuses++;

            if ( *lpEnd )
                lpEnd++;    // Move beyond the comma.
            else
                return;
            lpStart = lpEnd;
        }
    }

    /* Table has been populated so there might be more buses. */
    if ( *lpEnd )
    {
        m_uNextBusOffset = lpEnd - lpTimes;
        /* Make the Later button visible. */
        GetDlgItem( IDC_LATER_BTN )->EnableWindow( TRUE );
    }
    else
        /* At the end of the string therefore no more buses so hide the 'Later' button. */
        GetDlgItem( IDC_LATER_BTN )->EnableWindow( FALSE );

    if ( ! uNumBuses )
    {
        CString csText, csDay;
        // The string ID is in the day combo box item data.
        csDay.LoadString( m_DayCombo.GetItemData( m_nDayCombo ) );

        if ( ( m_uNextBusOffset == FIRST )  || ( lstrlen( lpTimes ) == 0 ) )
        {
            csText.Format( IDS_NO_SERVICE, m_aRoutes[ m_uRoute ].lpLine[ ROUTE_NUM_OFFSET ], csDay );
            MessageBox( csText );
        }
        else
        {
            csText.Format( IDS_NO_MORE_BUSES, m_aRoutes[ m_uRoute ].lpLine[ ROUTE_NUM_OFFSET ], csDay );
            m_uNextBusOffset = FIRST;
            MessageBox( csText );
            ReloadTimetable();
        }
    }
}

void CBusSchedWCEDlg::OnLButtonDown(UINT nFlags, CPoint point) 
{
    CMenu menu;
    VERIFY(menu.LoadMenu(IDR_MENU));
    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);
    pPopup->TrackPopupMenu(TPM_LEFTALIGN, point.x, point.y, AfxGetMainWnd());   

   CDialog::OnLButtonDown(nFlags, point);
}

void CBusSchedWCEDlg::OnSelectTimetable() 
{
    CFileDialog dlgDBFile( TRUE, _T( ".txt" ), NULL, OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST, _T( "Text files (*.txt)|*.txt" ), this );
    if ( dlgDBFile.DoModal() == IDOK )
    {
        m_dbFileName = dlgDBFile.GetPathName();
        UINT uNumItems = m_RouteCombo.GetCount();
        // Clear out the route list to make way for routes in new db.
        for ( INT i = uNumItems; i >= 0; i-- )
            m_RouteCombo.DeleteString( i );
        ReadTimetableFile();
        ReloadTimetable();
    }
}

void CBusSchedWCEDlg::OnAboutbox() 
{
    CAboutDlg dlgAbout;
    dlgAbout.DoModal();
    //OnOK();
}

void CBusSchedWCEDlg::OnPreferences() 
{
    CPrefsDlg dlgPrefs( m_uMinsFromTerminus );
    // Only save value if OK pressed.
    if ( dlgPrefs.DoModal() == IDOK )
    {
        m_uMinsFromTerminus = dlgPrefs.m_uMinsFromTerminus;
        m_uNextBusOffset = NEXT;
        ReloadTimetable();
    }
    //MessageBox( _T( "OnPreferences()" ) );
}

void CBusSchedWCEDlg::DisplayError(CONST DWORD dwErrorNumber)
{
    LPVOID lpMessageBuffer;

    ::FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                    NULL, dwErrorNumber, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                    (LPTSTR) &lpMessageBuffer, 0, NULL );
 
    MessageBox( ( LPTSTR ) lpMessageBuffer );

    // Free the buffer allocated by the system
    LocalFree( lpMessageBuffer );

    OnOK();
}

void CBusSchedWCEDlg::OnPaint() 
{
   CPaintDC dc(this); // device context for painting
   
   // TODO: Add your message handler code here
    // Load timetable as dialog is first painted. This allows error messages
    // to appear on top of the dialog instead of alone, without any dialog.
    if ( m_bFirstPaint )
    {
        m_bFirstPaint  = FALSE;

        if ( m_dbFileName.GetLength() == 0 )
        {
            // Reset route and direction when selecting new file.
            m_uRoute = 0;
            bDirection = DIRECTION_OUT;

            CString sMsg;
            sMsg.LoadString( IDS_NO_FILE_PREF );
            MessageBox( sMsg );
            OnSelectTimetable();
            if ( m_dbFileName.GetLength() == 0 )
            {
                MessageBox( _T( "BusSched cannot continue without a data file. Quitting." ) );
                OnOK();
            }
        }
        if ( ReadTimetableFile() )
        {
            // On first read set route combo to stored value if it is valid.
            if ( m_uRoute >= (UINT) m_RouteCombo.GetCount() )
                m_uRoute = 0;
            m_RouteCombo.SetCurSel( m_uRoute );
            ReloadTimetable();
        }
    }
   
   // Do not call CDialog::OnPaint() for painting messages
}
